# mypy: ignore-errors
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html


class UserURLMixin:  # pylint: disable=too-few-public-methods,no-self-use
    def user_url(self, obj):
        if not obj.user:
            return "-"
        url = reverse("admin:users_user_change", args=[obj.user.id])
        return format_html(
            '<a href="{}">{}</a>', url, obj.user.email or obj.user.username
        )

    user_url.short_description = "User"  # type: ignore


# pylint: disable=too-few-public-methods
class LinkRelatedMixin:
    def __init__(self, *args, **kwargs):
        self.original_list_display = self.list_display
        self.list_display = [
            entry for entry in self.list_display if not entry.endswith("__link")
        ]
        super().__init__(*args, **kwargs)
        for entry in self.original_list_display:
            if entry.endswith("__link"):
                self.add_foreign_key_link(entry)
        self.list_display = self.original_list_display

    def add_foreign_key_link(self, link_name):
        field_name = link_name.replace("__link", "")
        model_field = getattr(self.model, field_name).field
        related_fields = model_field.resolve_related_fields()
        related_model = related_fields[0][0].related_model
        app = related_model._meta.app_label
        name = related_model._meta.verbose_name

        @admin.display(description=field_name.capitalize())
        def fn(obj):
            related_instance = getattr(obj, field_name)
            url = reverse(f"admin:{app}_{name}_change", args=[related_instance.id])
            return format_html(
                '<a class="related-link" href="{}">{}</a>', url, related_instance
            )

        setattr(self, f"{link_name}", fn)



#!/usr/bin/env python
import re
import subprocess
import sys

import click


@click.command(
    context_settings=dict(
        ignore_unknown_options=True, allow_extra_args=True, help_option_names=["--noop"]
    ),
)
@click.argument("celery_args", nargs=-1, type=click.UNPROCESSED)
def command(celery_args):
    try:
        text = subprocess.check_output(grep_cmd(), shell=True, text=True)
    except subprocess.CalledProcessError as ex:
        if ex.returncode == 1:
            click.secho(
                "Error: Failed to automatically detect a celery app using :"
                f"cmd=`{grep_cmd()}`",
                fg="red",
            )
            raise click.Abort()

    lines = text.strip().split("\n", maxsplit=1)
    if len(lines) > 1:
        click.secho(
            ("Found multiple Celery apps. Unable to autodetect app. " f"Apps: {lines}"),
            fg="red",
        )
    celery_config_line = lines[0]
    celery_config_module = (
        celery_config_line.split(":")[0].replace("/", ".").replace(".py", "")
    )
    celery_app_var = re.search(r":(\w+)\s", celery_config_line).group(1)
    # pylint: disable=subprocess-run-check
    result = subprocess.run(
        ["celery", f"--app={celery_config_module}:{celery_app_var}"] + list(celery_args)
    )
    sys.exit(result.returncode)


def grep_cmd():
    return "grep -r --exclude-dir='.*' --include='*.py' '= Celery('"


if __name__ == "__main__":
    # pylint: disable=no-value-for-parameter
    command()



import djclick as click
from django.core import management
from iterfzf import iterfzf


def format_cmd(app, command_name):
    return f"{command_name:35s}{app}"


@click.command()
@click.option(
    "--fzf/--no-fzf",
    default=True,
    show_default=True,
)
def command(fzf):
    commands = management.get_commands()
    commands_list = [format_cmd(app, command) for command, app in commands.items()]
    if not fzf:
        click.echo("\n".join(commands_list))
        return
    chosen = iterfzf(iter(commands_list))
    if not chosen:
        return
    chosen = chosen.split()[0]
    click.echo(f"./manage.py {chosen}", nl=False)



from django.apps import apps
from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

User = get_user_model()


class Command(BaseCommand):
    help = "List installed django apps"

    def handle(self, *args, **options):
        for user in User.objects.all():
            print(user.username)



from pathlib import Path

import djclick as click
from django.apps import apps
from iterfzf import iterfzf


@click.command()
def command():
    d = local_apps()
    selected_app_label = iterfzf(iter(d.keys()))
    if not selected_app_label:
        return
    selected_app = d[selected_app_label]
    command_name = click.prompt("Command name")
    if not command_name.endswith(".py"):
        command_name += ".py"

    path = Path(selected_app.path) / "management" / "commands" / command_name
    path.parent.mkdir(parents=True, exist_ok=True)
    with open(path, "w") as f:
        f.write(
            """
import djclick as click

@click.command()
def command():
    click.secho("Hello", fg="green")
"""
        )
    click.secho(f"Created {path}", fg="green", bold=True)


def local_apps():
    d = {}
    for app in apps.get_app_configs():
        if ".venv" in app.path or ".virtualenv" in app.path:
            continue
        d[app.label] = app
    return d



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_toolshed', 'django_toolshed.management.commands']

package_data = \
{'': ['*'],
 'django_toolshed': ['static/django_toolshed/css/*',
                     'static/django_toolshed/images/*',
                     'static/django_toolshed/js/*',
                     'templates/django_toolshed/*']}

install_requires = \
['Django>=3.1,<4.0',
 'django-click>=2.2.0,<3.0.0',
 'ipython_genutils>=0.2.0,<0.3.0',
 'iterfzf>=0.5.0,<0.6.0']

entry_points = \
{'console_scripts': ['celery-auto-app = '
                     'django_toolshed.celery_auto_app:command']}

setup_kwargs = {
    'name': 'django-toolshed',
    'version': '0.4.0',
    'description': '',
    'long_description': None,
    'author': 'Dani Hodovic',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.apps import apps
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "List installed django apps"

    def handle(self, *args, **options):
        for app in apps.get_app_configs():
            print(app.label)



from django.apps import AppConfig


class DjangoToolsConfig(AppConfig):
    name = "django_toolshed"

    def ready(self):
        try:
            # pylint: disable=unused-import,import-outside-toplevel
            import django_toolshed.signals
        except ImportError:
            pass
